// file: ~kaldindi/work/class/ppm/ppm_read_1.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "ppm.h"

#include <math/integral_constants.h>
#include "ppm_constants.h"

// method: read_cc
//
// arguments:
//  uint_1* b_red_a: (output) intensity of the red component
//  uint_1* b_green_a: (output) intensity of the green component
//  uint_1* b_blue_a: (output) intensity of the blue component
//  int_4 offset_w_a: (input) starting pixel in the horizontal direction
//  int_4 offset_h_a: (input) starting pixel in the vertical direction
//  int_4 width_a: (input) number of pixels in the horizontal direction
//  int_4 height_a: (input) number of pixels in the vertical direction
//
// return: a logical_1 value indicating status
//
// this method reads data row by row. it uses a scratch buffer
// that is large enough to hold one row worth of data.
//
logical_1 Ppm::read_cc(uint_1* b_red_a, uint_1* b_green_a, uint_1* b_blue_a,
		       int_4 offset_w_a, int_4 offset_h_a,
		       int_4 width_a, int_4 height_a) {

  // check for out of bounds data in the width dimension
  //
  int_4 actual_width = width_a;

  if ((offset_w_a + actual_width) >= width_d) {
    actual_width = width_d - offset_w_a; 
  }
  if (actual_width < (int_4)0) {
    return ISIP_FALSE;
  }

  // check for out of bounds data in the height dimension
  //
  int_4 actual_height = height_a;

  if ((offset_h_a + actual_height) >= height_d) {
    actual_height = height_d - offset_h_a;
  }
  if (actual_height < (int_4)0) {
    return ISIP_FALSE;
  }

  // clear the output arrays
  //
  int_4 bsize = width_a * height_a * sizeof(uint_1);

  memset((void*)b_red_a, (int)0, (size_t)bsize);
  memset((void*)b_green_a, (int)0, (size_t)bsize);
  memset((void*)b_blue_a, (int)0, (size_t)bsize);

  // loop over all rows
  //
  int_4 o_row = (int_4)0;
  int_4 last_row = offset_h_a + actual_height;

  for (int_4 row_num=offset_h_a; row_num<last_row; row_num++) {

    // compute the new position we need to seek to
    //
    int_4 new_pos = (row_num * width_d + offset_w_a) *
      PPM_NBYTES_PER_PIXEL + beg_d;

    // seek to this new location
    //
    if (fseek(fp_d, new_pos, SEEK_SET) < (int)0) {
      return ISIP_FALSE;
    }

    // read the data into a temporary buffer
    //
    int_4 nbytes_to_read = actual_width * PPM_NBYTES_PER_PIXEL;
    uint_1 tmp_buf[nbytes_to_read];

    // read the data: note that we don't check for an out of bounds
    // read. this is because many applications want to process the image
    // in sub-images that are not multiple dimensions of the image.
    //
    fread((void*)tmp_buf, nbytes_to_read, (size_t)1, fp_d);

    // split the data into separate buffers
    //
    uint_1* ptr = tmp_buf;
    int_4 o_offset = o_row * width_a;
    int_4 last_val = o_offset + actual_width;
  
    for (int_4 i=o_offset; i<last_val; i++) {
      
      // separate each pixel
      //
      b_red_a[i] = *ptr++;
      b_green_a[i] = *ptr++;
      b_blue_a[i] = *ptr++;
    }

    // increment the output row counter
    //
    o_row++;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}


